==> org-lsst-ccs-drivers-reb-1.3-sources.jar: org/lsst/ccs/drivers/reb/ImageClient.java <==
package org.lsst.ccs.drivers.reb;

/**
 ***************************************************************************
 **
 **  Java interface to the image client routines
 **
 **  @author Owen Saxton
 **
 ***************************************************************************
 */
public class ImageClient {

   /**
    ***************************************************************************
    **
    **  Defines the data listener.
    **
    ***************************************************************************
    */
    public interface Listener {

       /**
        ***********************************************************************
        **
        **  Receives image data.
        **
        ***********************************************************************
        */
        public void processImage(Image image);

    }


   /**
    ***************************************************************************
    **
    **  Thread that obtains the image data for the listener.
    **
    ***************************************************************************
    */
    public class ReadThread extends Thread {

        Listener listener;   // Attached listener
        Image listenImage;   // Image supplied by listener
        int state = RUNNING;


       /**
        ***********************************************************************
        **
        **  Constructor.
        **
        ***********************************************************************
        */
        public ReadThread(Listener lstnr, Image image)
        {
            listener = lstnr;
            listenImage = image;
        }


       /**
        ***********************************************************************
        **
        **  Receives image data.
        **
        ***********************************************************************
        */
        @Override
        public void run()
        {
            while (state == RUNNING) {
                Image image = waitForImage(listenImage);
                if (state != RUNNING) break;
                getImage(image);
                Listener l = listener;
                if (state != RUNNING) break;
                l.processImage(image);
            }
            if (state == CLOSING) {
                deleteImageClient();
            }
        }


       /**
        ***********************************************************************
        **
        **  Sets the thread state.
        **
        ***********************************************************************
        */
        public void setState(int newState)
        {
            state = newState;
        }

    }


   /**
    ***************************************************************************
    **
    **  Private data.
    **
    ***************************************************************************
    */
    private final static int
        RUNNING = 0,
        CLOSING = 1,
        ENDING  = 2;

    private long client, subscriber;
    private ReadThread reader;
    private Listener imgListener;
    private Image listenImage;


   /**
    ***************************************************************************
    **
    **  Static initializer.
    **
    ***************************************************************************
    */
    static {
        System.loadLibrary("Reb");
        initSys();
    }


   /**
    ***************************************************************************
    **
    **  Finalizer.
    **
    ***************************************************************************
    */
    @Override
    protected void finalize() throws Throwable
    {
        super.finalize();
        deleteImageClient();
    }


   /**
    ***************************************************************************
    **
    **  Opens a connection using the default network interface.
    **
    **  @param  id  The ID of the REB to connect to
    **
    **  @exception  REBException 
    **
    ***************************************************************************
    */
    public void open(int id) throws REBException
    {
        open(id, null);
    }


   /**
    ***************************************************************************
    **
    **  Opens a connection using the specified network interface.
    **
    **  @param  id   The ID of the REB to connect to
    **
    **  @param  ifc  The name of the network interface to use.  If null or
    **               empty, the default interface is used.
    **
    **  @exception  REBException 
    **
    ***************************************************************************
    */
    public void open(int id, String ifc) throws REBException
    {
        if (client != 0) {
            throw new REBException("Image connection already open");
        }
        newImageClient(id, ifc);
        if (imgListener != null) {
            reader = new ReadThread(imgListener, listenImage);
            reader.setDaemon(true);
            reader.start();
        }
    }


   /**
    ***************************************************************************
    **
    **  Closes a connection.
    **
    **  @exception  REBException 
    **
    ***************************************************************************
    */
    public void close() throws REBException
    {
        checkOpen();
        if (reader != null) {
            reader.setState(ENDING);
            reader = null;
        }
        deleteImageClient();
    }


   /**
    ***************************************************************************
    **
    **  Awaits an image.
    **
    **  Waits until a new image has been generated.
    **
    **  @param  image  An Image object in which to save the reference and
    **                 metadata for the new image, or null if a new image
    **                 object is to be created.
    **
    **  @return  The Image object containing the new image reference and
    **           metadata.
    **
    **  @exception  REBException 
    **
    ***************************************************************************
    */
    public Image awaitImage(Image image) throws REBException
    {
        checkOpen();
        return waitForImage(image);
    }


   /**
    ***************************************************************************
    **
    **  Reads an image.
    **
    **  Gets the pixel data for an image.
    **
    **  @param  image  The Image object containing the valid metadata for an
    **                 image.  If the contained pixel data byte array is not
    **                 null and is large enough, the pixel data us stored
    **                 there.  Otherwise a new array is created.
    **
    **  @return  True if the pixel data was successfully read, false otherwise.
    **
    **  @exception  REBException 
    **
    ***************************************************************************
    */
    public boolean readImage(Image image) throws REBException
    {
        checkOpen();
        return getImage(image);
    }


   /**
    ***************************************************************************
    **
    **  Deletes an image's metadata reference.
    **
    **  Deletes referenced C++ object.
    **
    **  @param  image  The image whose metadata reference is to be deleted.
    **
    ***************************************************************************
    */
    public native static void deleteImageMetadataRef(Image image);


   /**
    ***************************************************************************
    **
    **  Resets the front end system.
    **
    ***************************************************************************
    */
    public void resetFrontEnd() throws REBException
    {
        checkOpen();
        reset();
    }


   /**
    ***************************************************************************
    **
    **  Sets the data listener.
    **
    **  Sets the data listener object, whose class must implement the Listener
    **  interface, and therefore must contain the processImage method.  This
    **  method is called with a complete Image object as its argument whenever
    **  a new image becomes available.
    **
    **  @param  listener  The listener object to be set as the listener.  Any
    **                    existing listener is replaced.
    **
    **  @param  image     An image object to be used to contain received
    **                    images, or null if a new object is to be created
    **                    each time.
    **
    ***************************************************************************
    */
    public void setListener(Listener listener, Image image)
    {
        clearListener();
        imgListener = listener;
        listenImage = image;
        if (client != 0) {
            reader = new ReadThread(listener, image);
            reader.setDaemon(true);
            reader.start();
        }
    }


   /**
    ***************************************************************************
    **
    **  Clears the data listener.
    **
    **  Clears the data listener object if it exists.
    **
    ***************************************************************************
    */
    public void clearListener()
    {
        if (reader != null) {
            reader.setState(ENDING);
            reader = null;
        }
        imgListener = null;
        listenImage = null;
    }


   /**
    ***************************************************************************
    **
    **  Performs one-time initialization.
    **
    **  Caches various field IDs for subsequent use.
    **
    ***************************************************************************
    */
    private native static void initSys();


   /**
    ***************************************************************************
    **
    **  Creates a new object.
    **
    **  Creates needed C++ objects.
    **
    **  @param  id   The ID of the REB to connect to
    **
    **  @param  ifc  The name of the network interface to use.  If null or
    **               empty, the default interface is used.
    **
    ***************************************************************************
    */
    private native void newImageClient(int id, String ifc);


   /**
    ***************************************************************************
    **
    **  Deletes an object.
    **
    **  Deletes referenced C++ objects.
    **
    ***************************************************************************
    */
    private native void deleteImageClient();


   /**
    ***************************************************************************
    **
    **  Awaits an image.
    **
    **  Waits until a new image has been generated.
    **
    **  @param  image  An Image object in which to save the reference and
    **                 metadata for the new image, or null if a new image
    **                 object is to be created.
    **
    **  @return  The Image object containing the new image reference and
    **           metadata.
    **
    ***************************************************************************
    */
    private native Image waitForImage(Image image);


   /**
    ***************************************************************************
    **
    **  Reads an image.
    **
    **  Gets the pixel data for an image.
    **
    **  @param  image  The Image object containing the valid metadata for an
    **                 image.  If the contained pixel data byte array is not
    **                 null and is large enough, the pixel data us stored
    **                 there.  Otherwise a new array is created.
    **
    **  @return  True if the pixel data was successfully read, false otherwise.
    **
    ***************************************************************************
    */
    private native boolean getImage(Image image);


   /**
    ***************************************************************************
    **
    **  Resets the front end system.
    **
    ***************************************************************************
    */
    private native void reset();


   /**
    ***************************************************************************
    **
    **  Checks that connection is open
    **
    **  @exception  REBException 
    **
    ***************************************************************************
    */
    private void checkOpen() throws REBException
    {
        if (client == 0) {
            throw new REBException("Image connection not open");
        }
    }

}

==> org-lsst-ccs-drivers-reb-1.4.1-sources.jar: org/lsst/ccs/drivers/reb/ImageClient.java <==
package org.lsst.ccs.drivers.reb;

/**
 ******************************************************************************
 **
 **  Java interface to the image client routines
 **
 **  @author Owen Saxton
 **
 ******************************************************************************
 */
public class ImageClient {

   /**
    ***************************************************************************
    **
    **  Defines the data listener.
    **
    ***************************************************************************
    */
    public interface Listener {

       /**
        ***********************************************************************
        **
        **  Receives image data.
        **
        **  @param  image  The received image
        **
        ***********************************************************************
        */
        public void processImage(Image image);

    }


   /**
    ***************************************************************************
    **
    **  Thread that obtains the image data for the listener.
    **
    ***************************************************************************
    */
    public class ReadThread extends Thread {

        Listener listener;   // Attached listener
        Image listenImage;   // Image supplied by listener
        int state = RUNNING;


       /**
        ***********************************************************************
        **
        **  Constructor.
        **
        **  @param  lstnr  The listener
        **
        **  @param  image  The image to use
        **
        ***********************************************************************
        */
        public ReadThread(Listener lstnr, Image image)
        {
            listener = lstnr;
            listenImage = image;
        }


       /**
        ***********************************************************************
        **
        **  Receives image data.
        **
        ***********************************************************************
        */
        @Override
        public void run()
        {
            while (state == RUNNING) {
                Image image = waitForImage(listenImage);
                if (state != RUNNING) break;
                getImage(image);
                Listener l = listener;
                if (state != RUNNING) break;
                l.processImage(image);
            }
            if (state == CLOSING) {
                deleteImageClient();
            }
        }


       /**
        ***********************************************************************
        **
        **  Sets the thread state.
        **
        **  @param  newState  The state to set
        **
        ***********************************************************************
        */
        public void setState(int newState)
        {
            state = newState;
        }

    }


   /**
    ***************************************************************************
    **
    **  Private data.
    **
    ***************************************************************************
    */
    private final static int
        RUNNING = 0,
        CLOSING = 1,
        ENDING  = 2;

    private long client, subscriber;
    private ReadThread reader;
    private Listener imgListener;
    private Image listenImage;


   /**
    ***************************************************************************
    **
    **  Static initializer.
    **
    ***************************************************************************
    */
    static {
        System.loadLibrary("Reb");
        initSys();
    }


   /**
    ***************************************************************************
    **
    **  Finalizer.
    **
    **  @throws  Throwable
    **
    ***************************************************************************
    */
    @Override
    protected void finalize() throws Throwable
    {
        super.finalize();
        deleteImageClient();
    }


   /**
    ***************************************************************************
    **
    **  Opens a connection using the default network interface.
    **
    **  @param  id  The ID of the REB to connect to
    **
    **  @exception  REBException 
    **
    ***************************************************************************
    */
    public void open(int id) throws REBException
    {
        open(id, null);
    }


   /**
    ***************************************************************************
    **
    **  Opens a connection using the specified network interface.
    **
    **  @param  id   The ID of the REB to connect to
    **
    **  @param  ifc  The name of the network interface to use.  If null or
    **               empty, the default interface is used.
    **
    **  @exception  REBException 
    **
    ***************************************************************************
    */
    public void open(int id, String ifc) throws REBException
    {
        if (client != 0) {
            throw new REBException("Image connection already open");
        }
        newImageClient(id, ifc);
        if (imgListener != null) {
            reader = new ReadThread(imgListener, listenImage);
            reader.setDaemon(true);
            reader.start();
        }
    }


   /**
    ***************************************************************************
    **
    **  Closes a connection.
    **
    **  @exception  REBException 
    **
    ***************************************************************************
    */
    public void close() throws REBException
    {
        checkOpen();
        if (reader != null) {
            reader.setState(ENDING);
            reader = null;
        }
        deleteImageClient();
    }


   /**
    ***************************************************************************
    **
    **  Awaits an image.
    **
    **  Waits until a new image has been generated.
    **
    **  @param  image  An Image object in which to save the reference and
    **                 metadata for the new image, or null if a new image
    **                 object is to be created.
    **
    **  @return  The Image object containing the new image reference and
    **           metadata.
    **
    **  @exception  REBException 
    **
    ***************************************************************************
    */
    public Image awaitImage(Image image) throws REBException
    {
        checkOpen();
        return waitForImage(image);
    }


   /**
    ***************************************************************************
    **
    **  Reads an image.
    **
    **  Gets the pixel data for an image.
    **
    **  @param  image  The Image object containing the valid metadata for an
    **                 image.  If the contained pixel data byte array is not
    **                 null and is large enough, the pixel data us stored
    **                 there.  Otherwise a new array is created.
    **
    **  @return  True if the pixel data was successfully read, false otherwise.
    **
    **  @exception  REBException 
    **
    ***************************************************************************
    */
    public boolean readImage(Image image) throws REBException
    {
        checkOpen();
        return getImage(image);
    }


   /**
    ***************************************************************************
    **
    **  Deletes an image's metadata reference.
    **
    **  Deletes referenced C++ object.
    **
    **  @param  image  The image whose metadata reference is to be deleted.
    **
    ***************************************************************************
    */
    public native static void deleteImageMetadataRef(Image image);


   /**
    ***************************************************************************
    **
    **  Resets the front end system.
    **
    **  @exception  REBException 
    **
    ***************************************************************************
    */
    public void resetFrontEnd() throws REBException
    {
        checkOpen();
        reset();
    }


   /**
    ***************************************************************************
    **
    **  Sets the data listener.
    **
    **  Sets the data listener object, whose class must implement the Listener
    **  interface, and therefore must contain the processImage method.  This
    **  method is called with a complete Image object as its argument whenever
    **  a new image becomes available.
    **
    **  @param  listener  The listener object to be set as the listener.  Any
    **                    existing listener is replaced.
    **
    **  @param  image     An image object to be used to contain received
    **                    images, or null if a new object is to be created
    **                    each time.
    **
    ***************************************************************************
    */
    public void setListener(Listener listener, Image image)
    {
        clearListener();
        imgListener = listener;
        listenImage = image;
        if (client != 0) {
            reader = new ReadThread(listener, image);
            reader.setDaemon(true);
            reader.start();
        }
    }


   /**
    ***************************************************************************
    **
    **  Clears the data listener.
    **
    **  Clears the data listener object if it exists.
    **
    ***************************************************************************
    */
    public void clearListener()
    {
        if (reader != null) {
            reader.setState(ENDING);
            reader = null;
        }
        imgListener = null;
        listenImage = null;
    }


   /**
    ***************************************************************************
    **
    **  Performs one-time initialization.
    **
    **  Caches various field IDs for subsequent use.
    **
    ***************************************************************************
    */
    private native static void initSys();


   /**
    ***************************************************************************
    **
    **  Creates a new object.
    **
    **  Creates needed C++ objects.
    **
    **  @param  id   The ID of the REB to connect to
    **
    **  @param  ifc  The name of the network interface to use.  If null or
    **               empty, the default interface is used.
    **
    ***************************************************************************
    */
    private native void newImageClient(int id, String ifc);


   /**
    ***************************************************************************
    **
    **  Deletes an object.
    **
    **  Deletes referenced C++ objects.
    **
    ***************************************************************************
    */
    private native void deleteImageClient();


   /**
    ***************************************************************************
    **
    **  Awaits an image.
    **
    **  Waits until a new image has been generated.
    **
    **  @param  image  An Image object in which to save the reference and
    **                 metadata for the new image, or null if a new image
    **                 object is to be created.
    **
    **  @return  The Image object containing the new image reference and
    **           metadata.
    **
    ***************************************************************************
    */
    private native Image waitForImage(Image image);


   /**
    ***************************************************************************
    **
    **  Reads an image.
    **
    **  Gets the pixel data for an image.
    **
    **  @param  image  The Image object containing the valid metadata for an
    **                 image.  If the contained pixel data byte array is not
    **                 null and is large enough, the pixel data us stored
    **                 there.  Otherwise a new array is created.
    **
    **  @return  True if the pixel data was successfully read, false otherwise.
    **
    ***************************************************************************
    */
    private native boolean getImage(Image image);


   /**
    ***************************************************************************
    **
    **  Resets the front end system.
    **
    ***************************************************************************
    */
    private native void reset();


   /**
    ***************************************************************************
    **
    **  Checks that connection is open
    **
    **  @exception  REBException 
    **
    ***************************************************************************
    */
    private void checkOpen() throws REBException
    {
        if (client == 0) {
            throw new REBException("Image connection not open");
        }
    }

}
